import * as React from 'react';
import { FieldEntity, FormInstance, Meta, NamePath, NotifyInfo, Rule, Store, ValidateOptions, InternalFormInstance } from './interface';
interface ChildProps {
    value?: any;
    onChange?: (...args: any[]) => void;
    onFocus?: (...args: any[]) => void;
    onBlur?: (...args: any[]) => void;
}
export interface FieldProps {
    name?: NamePath;
    children?: React.ReactElement | ((control: ChildProps, meta: Meta, form: FormInstance) => React.ReactNode);
    rules?: Rule[];
    /**
     * Set up `dependencies` field.
     * When dependencies field update and current field is touched, will trigger validate rules and render.
     */
    dependencies?: NamePath[];
    trigger?: string;
    validateTrigger?: string | string[];
    shouldUpdate?: (prevValues: any, nextValues: any, info: {
        source?: string;
    }) => boolean;
}
export interface FieldState {
    reset: boolean;
}
declare class Field extends React.Component<FieldProps, FieldState> implements FieldEntity {
    static contextType: React.Context<InternalFormInstance>;
    static defaultProps: {
        trigger: string;
        validateTrigger: string;
    };
    state: {
        reset: boolean;
    };
    private cancelRegisterFunc;
    /**
     * Follow state should not management in State since it will async update by React.
     * This makes first render of form can not get correct state value.
     */
    private touched;
    private validatePromise;
    private prevErrors;
    private prevValidating;
    componentDidMount(): void;
    componentWillUnmount(): void;
    cancelRegister: () => void;
    getNamePath: () => (string | number)[];
    refresh: () => void;
    onStoreChange: (prevStore: any, namePathList: (string | number)[][], info: NotifyInfo) => void;
    isFieldTouched: () => boolean;
    validateRules: (options?: ValidateOptions) => Promise<string[]>;
    isFieldValidating: () => boolean;
    getMeta: () => Meta;
    getOnlyChild: (children: string | number | boolean | {} | React.ReactElement<any, string | ((props: any) => React.ReactElement<any, string | any | (new (props: any) => React.Component<any, any, any>)>) | (new (props: any) => React.Component<any, any, any>)> | React.ReactNodeArray | React.ReactPortal | ((control: ChildProps, meta: Meta, context: any) => React.ReactNode)) => {
        child: React.ReactElement<any, string | ((props: any) => React.ReactElement<any, string | any | (new (props: any) => React.Component<any, any, any>)>) | (new (props: any) => React.Component<any, any, any>)>;
        isFunction: boolean;
    };
    getValue: (store?: Store) => any;
    getControlled: (childProps?: ChildProps) => {
        value: any;
        onChange?: (...args: any[]) => void;
        onFocus?: (...args: any[]) => void;
        onBlur?: (...args: any[]) => void;
    };
    render(): React.ReactElement<any, string | ((props: any) => React.ReactElement<any, string | any | (new (props: any) => React.Component<any, any, any>)>) | (new (props: any) => React.Component<any, any, any>)> | ((control: ChildProps, meta: Meta, form: FormInstance) => React.ReactNode);
}
export default Field;
